//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This shows how to make your code frame rate
///			independant. If you change the window size the 
///			object will still spin at the same speed.
//------------------------------------------------------------

#include <stdlib.h>
#include <GL/glut.h>

/// a structure to hold a mouse click
struct Click {

	/// the position of the mouse click
	int position[2];

	/// constructor
	Click(int x,int y) {

		// set position
		position[0] = x;
		position[1] = y;

		// set next pointer to NULL
		next = 0;
	}

	/// pointer to the next click
	Click* next;
};

/// the first and last elements in the queue.
Click* first=0;
Click* last=0;

//------------------------------------------------------------	Enqueue()
//
//	adds a new element to the end of the queue
//
void Enqueue(int x,int y) {

	// create a new click
	Click* p = new Click(x,y);

	// if queue if empty, set first and last to the new item
	if(!first)
		last = first = p;
	else
	// otherwise just insert into back of queue
	{
		last->next = p;
		last = p;
	}

}

//------------------------------------------------------------	Dequeue()
//
//	removes an element from the front of the queue
//
void Dequeue() {

	// if queue's empty, nothing to remove
	if(!first)
		return;

	// store a reference to the element at the front of the queue
	Click* p = first;

	// make the first pointer point to the second item in the queue
	first = first->next;

	// now safe to delete the item.
	delete p;
}

//------------------------------------------------------------	OnMouseClick()
//
void OnMouseClick(int button,int state,int x,int y) {

	// if button pressed
	if(state == GLUT_DOWN) {

		// add the mouse x and y coord into the queue
		if (button == GLUT_LEFT_BUTTON)
			Enqueue(x,y);

		// dequeue an element
		if (button == GLUT_RIGHT_BUTTON)
			Dequeue();
	}
	glutPostRedisplay();
}

//------------------------------------------------------------	DrawClicks()
//
//	This function is used to draw all data stored in the queue
//
void DrawClicks() {

	glColor3f(0.5,0.5,1);

	// draw a line between all stored points
	glBegin(GL_LINE_STRIP);
	Click* curr = first;

	// while not at end of the queue
	while (curr) {
		glVertex2iv(curr->position);
		curr = curr->next;
	}
	glEnd();

	glColor3f(1,0.5,0);

	// draw all stored points
	curr = first;
	glBegin(GL_POINTS);
	while (curr) {
		glVertex2iv(curr->position);
		curr = curr->next;
	}
	glEnd();
}

//------------------------------------------------------------	Draw()
//
void OnDraw() {

	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// draw the queue data
	DrawClicks();

	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {
	// enable depth testing
	glEnable(GL_DEPTH_TEST);

	// make the points a little bigger for our control points
	glPointSize(2);
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {
}

//------------------------------------------------------------	OnReshape()
//
void OnReshape(int w, int h)
{
	// prevent division by zero when minimised
	if (h==0)
		h=1;

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use a perspective projection
	glOrtho(0,w,h,0,-100,100);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
}

//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	// initialise glut
	glutInit(&argc,argv);

	// request a depth buffer, RGBA display mode, and we want double buffering
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);

	// set the initial window size
	glutInitWindowSize(640,480);

	// create the window
	glutCreateWindow("A Basic Queue");

	// set the function to use to draw our scene
	glutDisplayFunc(OnDraw);

	// set the function to handle changes in screen size
	glutReshapeFunc(OnReshape);

	// set the idle callback
	glutMouseFunc(OnMouseClick);
	
	// run our custom initialisation
	OnInit();

	// set the function to be called when we exit
	atexit(OnExit);

	// this function runs a while loop to keep the program running.
	glutMainLoop();
	return 0;
}
